@@ -129,7 +129,7 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
 	fputs(_(" -F, --pidfile <file>      read PIDs from file\n"), fp);
 	fputs(_(" -L, --logpidfile          fail if PID file is not locked\n"), fp);
 	fputs(_(" --ns <PID>                match the processes that belong to the same\n"
-		"                           namespace as <pid> or 0 for all namespaces\n"), fp);
+		"                           namespace as <pid>\n"), fp);
 	fputs(_(" --nslist <ns,...>         list which namespaces will be considered for\n"
 		"                           the --ns option.\n"
 		"                           Available namespaces: ipc, mnt, net, pid, user, uts\n"), fp);
@@ -652,7 +652,7 @@ static struct el * select_procs (int *num)
 	closeproc (ptp);
 	*num = matches;
 
-	if ((!matches) && (!opt_full) && (strlen(opt_pattern) > 15))
+	if ((!matches) && (!opt_full) && opt_pattern && (strlen(opt_pattern) > 15))
 		xwarnx(_("pattern that searches for process name longer than 15 characters will result in zero matches\n"
 				 "Try `%s -f' option to match against the complete command line."),
 				 program_invocation_short_name);
@@ -861,6 +861,8 @@ static void parse_opts (int argc, char **argv)
  *			break; */
 		case NS_OPTION:
 			opt_ns_pid = atoi(optarg);
+			if (opt_ns_pid == 0)
+				usage ('?');
 			++criteria_count;
 			break;
 		case NSLIST_OPTION:
@@ -914,7 +916,6 @@ int main (int argc, char **argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
-	opt_ns_pid = getpid();
 	parse_opts (argc, argv);
 
 	procs = select_procs (&num);
